#
#  - ( .     )
#
proc do_get_card_number {} {
    global state
    global account
    global pin

    set account {}
    set pin {}
    set state do_first_authorize

    return 0
}

#
#   -   (-)
#
proc do_first_authorize {} {
    global state
    global pin
    global account
    global retry_count

    set ev [authorize $account $pin {} [ani] info]

    switch -regexp $ev {
    {authorized} {
	    set state do_get_dest
	}
    {authorization failed} {
	    set state end
	}
    default {
	    set state end
	}
    }
    return 0
}

#
#    
#
proc do_get_dest {} {
    global state
    global destination
    global retry_count
    global second_auth_failure_code
    global second_auth_failure_counter
    global pr_input

    # generate 2 second silence
    playTone None
    set event [startTimer 2 Timer]
    while { $event != "timeout" } {
	set event [startTimer $Timer(TimeLeft) Timer]
    }

    puts "\t\tplayTone started"
    playTone Dial

    set ev [promptAndCollect pr_input info]

    switch -regexp $ev {
    {collect success} {
	    set destination $info(digits)
	    set state do_second_authorize
	}
    {collect aborted} {
	    set state do_get_dest
	}
    {collect fail} {
	    incr second_auth_failure_counter
	    set second_auth_failure_code 1
	    if {$second_auth_failure_counter < $retry_count} {
		set state do_get_dest
	    } {
		set state end
	    }
	}
    {collect timeout} {
	    incr second_auth_failure_counter
	    set second_auth_failure_code 3
	    if {$second_auth_failure_counter < $retry_count} {
		set state do_get_dest
	    } {
		set state end
	    }
	}
    default {
	    set state end
	}
    }
    return 0
}

#
#   .     .
#
proc do_second_authorize {} {
    global state
    global destination
    global last_destination
    global account
    global pin
    global creditTime
    global second_auth_failure_code
    global second_auth_failure_counter
    global retry_count
    global no_time_limit

    switch -regexp $destination {
    {^81[0-9]+$} {
	    set destination 6666#[string range $destination 2 end]
	}

    {^[78][0-9]+$} {
	    set destination 6767#7[string range $destination 1 end]
	}
    {^2$} {
	    set destination $last_destination
	}
    default { }
    }

    set last_destination $destination

    #   
    set ev [authorize $account $pin $destination [ani] info]

    switch -regexp $ev {
    {authorized} {
	    if {[getVariable aaa h323-credit-time info] != 0} {
		puts "\t\t**** got h323-credit-time: $info(0)"
		set creditTime $info(0)
	    } {
		puts "\t\t**** can't get h323-credit-amount"
		set creditTime "uninitialized"
	    }

	    if {$creditTime == "uninitialized"} {
		set state end
	    } elseif {$creditTime == "unlimited"} {
		set no_time_limit 1
		set state do_place_call
	    } {
		set state do_place_call
	    }
	}
    {authorization failed} {
	    incr second_auth_failure_counter
	    set second_auth_failure_code 2

	    if {$second_auth_failure_counter < $retry_count} {
		set state do_get_dest
	    } {
		set state end
	    }
	}
    default {
	    set state end
	}
    }
    return 0
}

#
# 
#
proc do_place_call {} {
    global state
    global destination
    global account
    global second_auth_failure_counter
    global second_auth_failure_code

    set second_auth_failure_counter 0
    set second_auth_failure_code 0
    set callInfo(accountNum) $account

    set ev [placeCall $destination callInfo info]

    switch -regexp $ev {
    {active} {
	    set state do_active
	}
    {call fail} {
	    set state do_place_fail
	}
    default {
	    set state end
	}
    }
    return 0
}

#
#   ( )
#
proc do_active_notimer {} {
    global state

    set ev [waitEvent]

    while {$ev == "digit"} {
	set ev [waitEvent]
    }
    set state end
    return 0
}

#
#   ( )?
#
proc do_active_last_timer {} {
    global state
    global warnTime
    global creditTime
    global no_play

    if {$no_play == 1} {
	set ev [startTimer [expr {$creditTime - 1}] info]
    } {
	set ev [startTimer [expr {$warnTime - 4}] info]
    }

    while {$ev == "digit"} {
	set ev [startTimer $info(timeLeft) info]
    }

    switch -regexp $ev {
    {longpound} {
	    do_longpound
	    return 0
	}
    {timeout} {
	    clearOutgoingLeg info
	    set state do_out_of_time
	}
    default {
	    set state end
	}
    }
    return 0
}

#
#  longpound'
#
proc do_longpound {} {
    global state
    global no_play

    set no_play 0

    clearOutgoingLeg info
    playTone Disconnect
    startTimer 2 timer
    playTone None
    set state do_get_dest
    return 0
}

#
# dunno
#
proc do_active_timer {} {
    global state
    global warnTime
    global creditTime
    global no_play

    set delay [expr {$creditTime - $warnTime}]
    set ev [startTimer $delay info]

    while {$ev == "digit"} {
	set ev [startTimer $info(timeLeft) info]
    }

    switch -regexp $ev {
    {longpound} {
	    do_longpound
	    return 0
	}
    {outgoing disconnected} {
	    set state do_get_dest
	}
    {timeout} {
	    if {$no_play != 1} {
		set warnTime [string trimleft $warnTime]
		playTone Disconnect
		startTimer 1 timer
		playTone None
	    }
	    do_active_last_timer
	}
    default {
	    set state end
	}
    }
    return 0
}

#
# dunno
#
proc do_active {} {
    global state
    global creditTime
    global warnTime
    global no_time_limit

    if {($no_time_limit == 1) ||($creditTime == "unlimited") ||($creditTime == "uninitialized")} {
	do_active_notimer
    } {
	if {$creditTime < $warnTime} {
	    do_active_last_timer
	} {
	    do_active_timer
	}
    }
    return 0
}

#
#    
#
proc do_out_of_time {} {
    global state

    playTone Disconnect
    startTimer 2 timer
    playTone None
    set state end
    return 0
}

#
#   
#
proc do_place_fail {} {
    global state

    playTone "Out Of Service"
    startTimer 2 timer
    playTone None

    set state do_get_dest

    return 0
}

#-------------------------------------------------------
# main
#

acceptCall

# set primary language
setVariable language 1

# set "longpound" feature
setVariable longPound true

set second_auth_failure_code 0
set second_auth_failure_counter 0
set no_play 0
set dest_failure_code 0
set no_time_limit 0
set destination 0900405
set last_destination $destination

# collect digits, use dial plan, may be interrupted
set pr_input(interrupt) true
set pr_input(abortKey) *
set pr_input(terminationKey) #
set pr_input(dialPlan) true

# get global configuration parameters from AS5xxx
set retry_count [getVariable config retryCount]
set warnTime [getVariable config warningTime]

set state do_get_card_number

while {$state != "end"} {
    puts "************ cid([callID]) app running state $state"
    $state
}

